using System;
using System.ComponentModel;
using System.Xml.Serialization;

namespace EQIA.EQIDL
{
    /// <summary>
    /// Stores size information for a ui element.
    /// </summary>
    [TypeConverter(typeof(SizeConverter))]
    public struct Size
    {
        /// <summary>
        /// An empty size with both width and height set to zero.
        /// </summary>
        [XmlIgnore()]
        [Browsable(false)]
        public static readonly Size Empty = new Size(0, 0);

        private int myWidth;
        private int myHeight;

        /// <summary>
        /// Width in pixels.
        /// </summary>
        [XmlElement("CX")]
        [Browsable(true), Description("Width in pixels.")]
        [NotifyParentProperty(true)]
        public int Width { get { return myWidth; } set { myWidth = value; } }

        /// <summary>
        /// Height in pixels.
        /// </summary>
        [XmlElement("CY")]
        [Browsable(true), Description("Height in pixels.")]
        [NotifyParentProperty(true)]
        public int Height { get { return myHeight; } set { myHeight = value; } }

        ///// <summary>
        ///// Create a new size.
        ///// </summary>
        //public Size()
        //    : this(0, 0)
        //{
        //}

        /// <summary>
        /// Create a new size.
        /// </summary>
        /// <param name="width">Width in pixels.</param>
        /// <param name="height">Height in pixels.</param>
        public Size(int width, int height)
        {
            myWidth = width;
            myHeight = height;
        }

        public override string ToString()
        {
            return myWidth + ", " + myHeight;
        }

        public static bool operator !=(Size s1, Size s2)
        {
            return s1.GetHashCode() != s2.GetHashCode();
        }

        public static bool operator ==(Size s1, Size s2)
        {
            return s1.GetHashCode() == s2.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            return this == (Size)obj;
        }

        public override int GetHashCode()
        {
            return myWidth.GetHashCode() | myHeight.GetHashCode();
        }
    }

    /// <summary>
    /// Type converter that will convert a EQIA.EQIDL.Size to string and string to EQIA.EQIDL.Size.
    /// </summary>
    public class SizeConverter : ExpandableObjectConverter
    {
        public override bool CanConvertTo(ITypeDescriptorContext context, System.Type destinationType)
        {
            return (destinationType == typeof(Size)) ? 
                true : 
                base.CanConvertTo(context, destinationType);
        }

        public override bool CanConvertFrom(ITypeDescriptorContext context, System.Type sourceType)
        {
            return (sourceType == typeof(string)) ? 
                true : 
                base.CanConvertFrom(context, sourceType);
        }

        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, System.Type destinationType)
        {
            if (value != null && value.GetType() != typeof(Size))
                throw new Exception("'" + value.GetType() + "' can not be converted to type '" + typeof(Size) + "'.");

            return (destinationType == typeof(string)) ?
                ((Size)value).ToString() :
                base.ConvertTo(context, culture, value, destinationType);
        }

        public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {
            if (value.GetType() == typeof(string))
            {
                try
                {
                    string[] args = value.ToString().Split(',');
                    return new Size(int.Parse(args[0].Trim()), int.Parse(args[1].Trim()));
                }
                catch
                {
                    throw new Exception("Can not convert '" + value.ToString() + "' to type '" + typeof(Size) + "'.");
                }
            }

            return base.ConvertFrom(context, culture, value);
        }
    }
}
